@@ -178,7 +178,8 @@ Actions ClientStateMachine::processConnect(
   connect.extensions = extensions;
   connect.cachedPsk = std::move(cachedPsk);
   connect.echConfigs = std::move(echConfigs);
-  return detail::processEvent(state, std::move(connect));
+  fizz::Param p(std::move(connect));
+  return detail::processEvent(state, p);
 }
 
 Actions ClientStateMachine::processSocketData(
@@ -196,7 +197,7 @@ Actions ClientStateMachine::processSocketData(
     if (!param.has_value()) {
       return actions(WaitForData{param.sizeHint});
     }
-    return detail::processEvent(state, std::move(*param));
+    return detail::processEvent(state, *param);
   } catch (const FizzException& e) {
     return detail::handleError(
         state,
@@ -219,19 +220,22 @@ Actions ClientStateMachine::processSocketData(
 Actions ClientStateMachine::processWriteNewSessionTicket(
     const State& state,
     WriteNewSessionTicket write) {
-  return detail::processEvent(state, std::move(write));
+  fizz::Param p = std::move(write);
+  return detail::processEvent(state, p);
 }
 
 Actions ClientStateMachine::processAppWrite(
     const State& state,
     AppWrite write) {
-  return detail::processEvent(state, std::move(write));
+  fizz::Param p = std::move(write);
+  return detail::processEvent(state, p);
 }
 
 Actions ClientStateMachine::processEarlyAppWrite(
     const State& state,
     EarlyAppWrite write) {
-  return detail::processEvent(state, std::move(write));
+  fizz::Param p = std::move(write);
+  return detail::processEvent(state, p);
 }
 
 Actions ClientStateMachine::processAppClose(const State& state) {
@@ -245,16 +249,17 @@ Actions ClientStateMachine::processAppCloseImmediate(const State& state) {
 Actions ClientStateMachine::processKeyUpdateInitiation(
     const State& state,
     KeyUpdateInitiation keyUpdateInitiation) {
-  return detail::processEvent(state, std::move(keyUpdateInitiation));
+  fizz::Param p = std::move(keyUpdateInitiation);
+  return detail::processEvent(state, p);
 }
 
 namespace detail {
 
-Actions processEvent(const State& state, Param param) {
+Actions processEvent(const State& state, Param& param) {
   auto event = EventVisitor()(param);
   try {
     return sm::StateMachine<ClientTypes>::getHandler(state.state(), event)(
-        state, std::move(param));
+        state, param);
   } catch (const FizzException& e) {
     return detail::handleError(
         state,
@@ -330,7 +335,8 @@ Actions handleAppClose(const State& state) {
   }
 }
 
-Actions handleInvalidEvent(const State& state, Event event, Param param) {
+Actions handleInvalidEvent(const State& state, Param& param) {
+  auto event = EventVisitor()(param);
   if (event == Event::Alert) {
     auto& alert = *param.asAlert();
     throw FizzException(
@@ -629,11 +635,17 @@ static ech::SupportedECHConfig getSupportedECHConfig(
   // Convert vectors to use HPKE types.
   std::vector<hpke::KEMId> supportedKEMs(supportedGroups.size());
   for (const auto& group : supportedGroups) {
-    supportedKEMs.push_back(hpke::getKEMId(group));
+    const auto kemId = hpke::tryGetKEMId(group);
+    if (kemId.has_value()) {
+      supportedKEMs.push_back(*kemId);
+    }
   }
   std::vector<hpke::AeadId> supportedAeads(supportedCiphers.size());
   for (const auto& suite : supportedCiphers) {
-    supportedAeads.push_back(hpke::getAeadId(suite));
+    const auto aeadId = hpke::tryGetAeadId(suite);
+    if (aeadId.has_value()) {
+      supportedAeads.push_back(*aeadId);
+    }
   }
 
   // Get a supported ECH config.
@@ -715,7 +727,8 @@ static ClientHello constructEncryptedClientHello(
     hpke::SetupResult& hpkeSetup,
     const Random& outerRandom,
     Buf fakeSni,
-    const folly::Optional<ClientPresharedKey>& greasePsk) {
+    const folly::Optional<ClientPresharedKey>& greasePsk,
+    const std::vector<ExtensionType>& outerExtensionTypes) {
   DCHECK(e == Event::ClientHello || e == Event::HelloRetryRequest);
 
   // Outer chlo is missing the inner ECH. We also replace any PSK
@@ -736,7 +749,6 @@ static ClientHello constructEncryptedClientHello(
   // Substitute in outer random
   chloOuter.random = outerRandom;
 
-  Extension encodedECHExtension;
   // Create the encrypted client hello inner extension.
   switch (supportedConfig.config.version) {
     case (ech::ECHVersion::Draft15): {
@@ -747,14 +759,16 @@ static ClientHello constructEncryptedClientHello(
             std::move(chlo),
             chloOuter.clone(),
             hpkeSetup,
-            greasePsk);
+            greasePsk,
+            outerExtensionTypes);
       } else {
         clientECHExtension = encryptClientHelloHRR(
             supportedConfig,
             std::move(chlo),
             chloOuter.clone(),
             hpkeSetup,
-            greasePsk);
+            greasePsk,
+            outerExtensionTypes);
       }
       chloOuter.extensions.push_back(
           encodeExtension(std::move(clientECHExtension)));
@@ -771,7 +785,7 @@ static ClientHello constructEncryptedClientHello(
 Actions
 EventHandler<ClientTypes, StateEnum::Uninitialized, Event::Connect>::handle(
     const State& /*state*/,
-    Param param) {
+    Param& param) {
   auto& connect = *param.asConnect();
 
   auto context = std::move(connect.context);
@@ -921,11 +935,12 @@ EventHandler<ClientTypes, StateEnum::Uninitialized, Event::Connect>::handle(
         echParams->setupResult,
         random,
         echParams->fakeSni->clone(),
-        greasePsk);
+        greasePsk,
+        context->getECHOuterExtensionTypes());
 
     // Update SNI now
     echSni = std::move(sni);
-    sni = echParams->fakeSni->clone()->moveToFbString().toStdString();
+    sni = echParams->fakeSni->to<std::string>();
 
     // Save client hello inner
     encodedECH = std::move(encodedClientHello);
@@ -1171,7 +1186,7 @@ static NegotiatedPsk negotiatePsk(
 
 Actions
 EventHandler<ClientTypes, StateEnum::ExpectingServerHello, Event::ServerHello>::
-    handle(const State& state, Param param) {
+    handle(const State& state, Param& param) {
   auto shlo = std::move(*param.asServerHello());
 
   Protocol::checkAllowedExtensions(shlo, *state.requestedExtensions());
@@ -1463,7 +1478,7 @@ static std::map<NamedGroup, std::unique_ptr<KeyExchange>> getHrrKeyExchangers(
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingServerHello,
-    Event::HelloRetryRequest>::handle(const State& state, Param param) {
+    Event::HelloRetryRequest>::handle(const State& state, Param& param) {
   auto hrr = std::move(*param.asHelloRetryRequest());
 
   Protocol::checkAllowedExtensions(hrr, *state.requestedExtensions());
@@ -1623,7 +1638,8 @@ Actions EventHandler<
         state.echState()->hpkeSetup,
         *state.clientRandom(),
         folly::IOBuf::copyBuffer(*state.sni()),
-        greasePsk);
+        greasePsk,
+        state.context()->getECHOuterExtensionTypes());
 
     // Save client hello inner
     encodedECH = std::move(encodedClientHello);
@@ -1731,7 +1747,7 @@ static void validateAcceptedEarly(
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingEncryptedExtensions,
-    Event::EncryptedExtensions>::handle(const State& state, Param param) {
+    Event::EncryptedExtensions>::handle(const State& state, Param& param) {
   auto ee = std::move(*param.asEncryptedExtensions());
 
   Protocol::checkAllowedExtensions(ee, *state.requestedExtensions());
@@ -1746,8 +1762,7 @@ Actions EventHandler<
           "alpn list does not contain exactly one protocol",
           AlertDescription::illegal_parameter);
     }
-    appProto =
-        alpn->protocol_name_list.front().name->moveToFbString().toStdString();
+    appProto = alpn->protocol_name_list.front().name->to<std::string>();
     if (std::find(
             state.context()->getSupportedAlpns().begin(),
             state.context()->getSupportedAlpns().end(),
@@ -1846,7 +1861,7 @@ getClientCert(
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingCertificate,
-    Event::CertificateRequest>::handle(const State& state, Param param) {
+    Event::CertificateRequest>::handle(const State& state, Param& param) {
   if (state.clientAuthRequested()) {
     throw FizzException(
         "duplicate certificate request message",
@@ -1960,7 +1975,7 @@ static MutateState handleCertMsg(
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingCertificate,
-    Event::CompressedCertificate>::handle(const State& state, Param param) {
+    Event::CompressedCertificate>::handle(const State& state, Param& param) {
   if (state.context()->getSupportedCertDecompressionAlgorithms().empty()) {
     throw FizzException(
         "compressed certificate received unexpectedly",
@@ -1999,7 +2014,7 @@ Actions EventHandler<
 
 Actions
 EventHandler<ClientTypes, StateEnum::ExpectingCertificate, Event::Certificate>::
-    handle(const State& state, Param param) {
+    handle(const State& state, Param& param) {
   auto certMsg = std::move(*param.asCertificateMsg());
 
   state.handshakeContext()->appendToTranscript(*certMsg.originalEncoding);
@@ -2012,7 +2027,7 @@ EventHandler<ClientTypes, StateEnum::ExpectingCertificate, Event::Certificate>::
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingCertificateVerify,
-    Event::CertificateVerify>::handle(const State& state, Param param) {
+    Event::CertificateVerify>::handle(const State& state, Param& param) {
   auto certVerify = std::move(*param.asCertificateVerify());
 
   if (std::find(
@@ -2071,7 +2086,7 @@ Actions EventHandler<
 
 Actions
 EventHandler<ClientTypes, StateEnum::ExpectingFinished, Event::Finished>::
-    handle(const State& state, Param param) {
+    handle(const State& state, Param& param) {
   auto finished = std::move(*param.asFinished());
 
   if (state.readRecordLayer()->hasUnparsedHandshakeData()) {
@@ -2262,7 +2277,7 @@ static uint32_t getMaxEarlyDataSize(const NewSessionTicket& nst) {
 
 Actions
 EventHandler<ClientTypes, StateEnum::Established, Event::NewSessionTicket>::
-    handle(const State& state, Param param) {
+    handle(const State& state, Param& param) {
   auto nst = std::move(*param.asNewSessionTicket());
 
   auto derivedResumptionSecret = state.keyScheduler()->getResumptionSecret(
@@ -2296,7 +2311,7 @@ EventHandler<ClientTypes, StateEnum::Established, Event::NewSessionTicket>::
 Actions
 EventHandler<ClientTypes, StateEnum::Established, Event::AppData>::handle(
     const State&,
-    Param param) {
+    Param& param) {
   auto& appData = *param.asAppData();
 
   return actions(DeliverAppData{std::move(appData.data)});
@@ -2305,7 +2320,7 @@ EventHandler<ClientTypes, StateEnum::Established, Event::AppData>::handle(
 Actions
 EventHandler<ClientTypes, StateEnum::Established, Event::AppWrite>::handle(
     const State& state,
-    Param param) {
+    Param& param) {
   auto& appWrite = *param.asAppWrite();
 
   WriteToSocket write;
@@ -2319,7 +2334,7 @@ EventHandler<ClientTypes, StateEnum::Established, Event::AppWrite>::handle(
 
 Actions
 EventHandler<ClientTypes, StateEnum::Established, Event::KeyUpdateInitiation>::
-    handle(const State& state, Param param) {
+    handle(const State& state, Param& param) {
   if (state.readRecordLayer()->hasUnparsedHandshakeData()) {
     throw FizzException(
         "data after key_update", AlertDescription::unexpected_message);
@@ -2357,7 +2372,7 @@ EventHandler<ClientTypes, StateEnum::Established, Event::KeyUpdateInitiation>::
 Actions
 EventHandler<ClientTypes, StateEnum::Established, Event::KeyUpdate>::handle(
     const State& state,
-    Param param) {
+    Param& param) {
   auto& keyUpdate = *param.asKeyUpdate();
 
   if (state.readRecordLayer()->hasUnparsedHandshakeData()) {
@@ -2428,7 +2443,8 @@ EventHandler<ClientTypes, StateEnum::Established, Event::KeyUpdate>::handle(
 // action to invoke depends on how the higher layer will react to rejected
 // early data, we give the write back in a special action for the higher layer
 // to handle.
-static Actions ignoreEarlyAppWrite(const State& state, EarlyAppWrite write) {
+static Actions ignoreEarlyAppWrite(const State& state, Param& param) {
+  auto& write = *param.asEarlyAppWrite();
   if (*state.earlyDataType() != EarlyDataType::Rejected) {
     throw FizzException("ignoring valid early write", folly::none);
   }
@@ -2438,7 +2454,8 @@ static Actions ignoreEarlyAppWrite(const State& state, EarlyAppWrite write) {
   return actions(std::move(failedWrite));
 }
 
-static Actions handleEarlyAppWrite(const State& state, EarlyAppWrite appWrite) {
+static Actions handleEarlyAppWrite(const State& state, Param& param) {
+  auto& appWrite = *param.asEarlyAppWrite();
   if (state.context()->getOmitEarlyRecordLayer()) {
     throw FizzException("early app writes disabled", folly::none);
   }
@@ -2447,7 +2464,7 @@ static Actions handleEarlyAppWrite(const State& state, EarlyAppWrite appWrite) {
     case EarlyDataType::NotAttempted:
       throw FizzException("invalid early write", folly::none);
     case EarlyDataType::Rejected:
-      return ignoreEarlyAppWrite(state, std::move(appWrite));
+      return ignoreEarlyAppWrite(state, param);
     case EarlyDataType::Attempted:
     case EarlyDataType::Accepted: {
       WriteToSocket write;
@@ -2479,43 +2496,43 @@ static Actions handleEarlyAppWrite(const State& state, EarlyAppWrite appWrite) {
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingServerHello,
-    Event::EarlyAppWrite>::handle(const State& state, Param param) {
-  return handleEarlyAppWrite(state, std::move(*param.asEarlyAppWrite()));
+    Event::EarlyAppWrite>::handle(const State& state, Param& param) {
+  return handleEarlyAppWrite(state, param);
 }
 
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingEncryptedExtensions,
-    Event::EarlyAppWrite>::handle(const State& state, Param param) {
-  return handleEarlyAppWrite(state, std::move(*param.asEarlyAppWrite()));
+    Event::EarlyAppWrite>::handle(const State& state, Param& param) {
+  return handleEarlyAppWrite(state, param);
 }
 
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingCertificate,
-    Event::EarlyAppWrite>::handle(const State& state, Param param) {
-  return ignoreEarlyAppWrite(state, std::move(*param.asEarlyAppWrite()));
+    Event::EarlyAppWrite>::handle(const State& state, Param& param) {
+  return ignoreEarlyAppWrite(state, param);
 }
 
 Actions EventHandler<
     ClientTypes,
     StateEnum::ExpectingCertificateVerify,
-    Event::EarlyAppWrite>::handle(const State& state, Param param) {
-  return ignoreEarlyAppWrite(state, std::move(*param.asEarlyAppWrite()));
+    Event::EarlyAppWrite>::handle(const State& state, Param& param) {
+  return ignoreEarlyAppWrite(state, param);
 }
 
 Actions
 EventHandler<ClientTypes, StateEnum::ExpectingFinished, Event::EarlyAppWrite>::
-    handle(const State& state, Param param) {
-  return handleEarlyAppWrite(state, std::move(*param.asEarlyAppWrite()));
+    handle(const State& state, Param& param) {
+  return handleEarlyAppWrite(state, param);
 }
 
 Actions
 EventHandler<ClientTypes, StateEnum::Established, Event::EarlyAppWrite>::handle(
     const State& state,
-    Param param) {
-  auto appWrite = std::move(*param.asEarlyAppWrite());
+    Param& param) {
   if (*state.earlyDataType() == EarlyDataType::Accepted) {
+    auto appWrite = std::move(*param.asEarlyAppWrite());
     // It's possible that we had queued early writes before full handshake
     // success. It's fine to write them on the normal record layer as long as
     // the early data was accepted, otherwise we need to ignore them to preserve
@@ -2527,14 +2544,14 @@ EventHandler<ClientTypes, StateEnum::Established, Event::EarlyAppWrite>::handle(
     write.flags = appWrite.flags;
     return actions(std::move(write));
   } else {
-    return ignoreEarlyAppWrite(state, std::move(appWrite));
+    return ignoreEarlyAppWrite(state, param);
   }
 }
 
 Actions
 EventHandler<ClientTypes, StateEnum::Established, Event::CloseNotify>::handle(
     const State& state,
-    Param param) {
+    Param& param) {
   ensureNoUnparsedHandshakeData(state, Event::CloseNotify);
   auto& closenotify = *param.asCloseNotify();
   auto eod = EndOfData(std::move(closenotify.ignoredPostCloseData));
@@ -2556,7 +2573,7 @@ EventHandler<ClientTypes, StateEnum::Established, Event::CloseNotify>::handle(
 
 Actions
 EventHandler<ClientTypes, StateEnum::ExpectingCloseNotify, Event::CloseNotify>::
-    handle(const State& state, Param param) {
+    handle(const State& state, Param& param) {
   ensureNoUnparsedHandshakeData(state, Event::CloseNotify);
   auto& closenotify = *param.asCloseNotify();
   auto eod = EndOfData(std::move(closenotify.ignoredPostCloseData));
