@@ -20,27 +20,27 @@ namespace test {
 class HandshakeContextTest : public testing::Test {};
 
 TEST_F(HandshakeContextTest, TestHandshakeContextSingle) {
-  HandshakeContextImpl<Sha256> context(kHkdfLabelPrefix.str());
+  HandshakeContextImpl<Sha256> context;
   context.appendToTranscript(folly::IOBuf::copyBuffer("ClientHello"));
   context.getHandshakeContext();
 }
 
 TEST_F(HandshakeContextTest, TestHandshakeContextMultiple) {
-  HandshakeContextImpl<Sha256> context(kHkdfLabelPrefix.str());
+  HandshakeContextImpl<Sha256> context;
   context.appendToTranscript(folly::IOBuf::copyBuffer("ClientHello"));
   context.appendToTranscript(folly::IOBuf::copyBuffer("ServerHello"));
   context.getHandshakeContext();
 }
 
 TEST_F(HandshakeContextTest, TestFinished) {
-  HandshakeContextImpl<Sha256> context(kHkdfLabelPrefix.str());
+  HandshakeContextImpl<Sha256> context;
   context.appendToTranscript(folly::IOBuf::copyBuffer("ClientHello"));
   std::vector<uint8_t> baseKey(Sha256::HashLen);
   context.getFinishedData(range(baseKey));
 }
 
 TEST_F(HandshakeContextTest, TestEmpty) {
-  HandshakeContextImpl<Sha256> context(kHkdfLabelPrefix.str());
+  HandshakeContextImpl<Sha256> context;
   context.getHandshakeContext();
   std::array<uint8_t, Sha256::HashLen> key{4};
   context.getFinishedData(folly::range(key));
