@@ -8,13 +8,13 @@
 
 #pragma once
 
-#include <fizz/backend/openssl/OpenSSLFactory.h>
 #include <fizz/crypto/aead/test/Mocks.h>
 #include <fizz/crypto/exchange/test/Mocks.h>
 #include <fizz/crypto/test/Mocks.h>
 #include <fizz/protocol/AsyncFizzBase.h>
 #include <fizz/protocol/Certificate.h>
 #include <fizz/protocol/CertificateVerifier.h>
+#include <fizz/protocol/DefaultFactory.h>
 #include <fizz/protocol/HandshakeContext.h>
 #include <fizz/protocol/KeyScheduler.h>
 #include <fizz/protocol/Types.h>
@@ -215,7 +215,7 @@ class MockCertificateVerifier : public CertificateVerifier {
       (const));
 };
 
-class MockFactory : public openssl::OpenSSLFactory {
+class MockFactory : public ::fizz::DefaultFactory {
  public:
   MOCK_METHOD(
       std::unique_ptr<PlaintextReadRecordLayer>,
@@ -338,7 +338,7 @@ class MockFactory : public openssl::OpenSSLFactory {
   }
 };
 
-class MockAsyncKexFactory : public openssl::OpenSSLFactory {
+class MockAsyncKexFactory : public ::fizz::DefaultFactory {
  public:
   MOCK_METHOD(
       std::unique_ptr<KeyExchange>,
